SystemOrganization addCategory: #'Phidget-Core'!

!SequenceableCollection methodsFor: '*Phidget-Core' stamp: 'lk 6/16/2009 16:35'!
indexIsInRange: index
	"answer true if the given index is in range of this collection, false otherwise."

	^ index isInteger
		and: [ index >= 1 ]
		and: [ index <= self size ]! !

Object subclass: #PhdCrComponent
	instanceVariableNames: 'phidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrComponent commentStamp: 'lk 6/16/2009 16:45' prior: 0!
I am an abstract class for any Components a Phidget can have.


Instance Variables
	phidget:		<PhdCrPhidget>

phidget
	- the 'parent' of this component. Every component belongs to a Phidget. Only if the Phidget is attached the component is usable.
	- I need this value to invoke the primitives to talk with the plugin/Phidget hardware, because every primitive I know needs the Phidget's unique identifier (id).


I must be created with an instance of PhdCrPhidget as parameter that sets my 'phidget' instance variable (see 'class newWith:' and 'setParent').!

PhdCrComponent subclass: #PhdCrCompRFID
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompRFID commentStamp: 'lk 6/16/2009 19:45' prior: 0!
I am a special Phidget Component for reading RFID tags.
I have an antenna that can be enabled or disabled (getter/setter exist).
When my antenna is on I can read information about a near tag. Actually I only know which was the last tag I read, and whether I am currently reading any. The Phidget I belong to can signal when I found or lost a tag. (getter)

Currently I only belong to PhdCrRFID.
Example for usage there:
	rfid := PhdCrRFID new.
	rfid
		"The antenna might still be disabled, so enable it as soon as the Phidget is attached."
		on: #Attach do: [ rfid rfidSensor setAntennaState: true ];
		"Print to Transcript whenever we find or lose a tag. Print its identifier."
		on: #TagFound do: [ Transcript show: 'Tag found: ', rfid rfidSensor currentTag; cr ];
		on: #TagLost do: [ Transcript show: 'Tag lost: ', rfid rfidSensor lastTag; cr ];
		openAny.
	"Now attach a RFID Phidget, hold a tag near the Phidget and then take it away."
	rfid close.
!

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primGetLastTag: phidgetIdArray
	"Answer a array of 5 bytes representing the id of the tag last read by this phidget, or nil if there was an error."

	<primitive: 'primGetLastRFIDTag' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primHasTag: phidgetIdArray
	"Answer 1 if the Phidget currently reads a tag, or other values otherwise."
	
	<primitive: 'primHasRFIDTag' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primIsAntennaOn: phidgetIdArray
	"Answer 1 if the antenna is on, or other values otherwise."
	
	<primitive: 'primIsRFIDAntennaOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetAntennaState: phidgetIdArray to: state
	"Set the state of the antenna to on (state=1) or off (state=0).
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetRFIDAntennaState' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetSemaphoreIndexTagFound: semaphoreIndex for: phidgetIdArray
	"registers the given semaphore to be signaled when a tag is found by the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexRFIDTagFound' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetSemaphoreIndexTagLost: semaphoreIndex for: phidgetIdArray
	"registers the given semaphore to be signaled when a tag is lost by the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexRFIDTagLost' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:37'!
currentTag
	"Returns a string which represents a hex-value of the 5 bytes tag ID of the tag currently read.
	If there is no tag currently read, return an empty string."
	
	^ self hasTag
		ifTrue: [ self lastTag ]
		ifFalse: [ '' ]! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 17:19'!
currentTagBytes
	"Returns a byte array with the 5 elements of the tag ID currently read.
	If there is no tag currently read, return nil."
	
	^ self hasTag
		ifTrue: [ self lastTagBytes ]
		ifFalse: [ nil ]! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:34'!
hasTag
	"Returns true is a tag is currently read and false if not."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primHasTag: phidget id) == 1! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 17:25'!
isAntennaOn
	"Answer true if the antenna is on, false otherwise."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primIsAntennaOn: phidget id) == 1! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:13'!
isAntennaOn: aBoolean 
	"Sets the state of the antenna to true or false.
	The antenna must be enabled in order to find tags near the Phidget.
	Returns true if setting the antenna state was successful, false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetAntennaState: phidget id
		to: anInt) == 1! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:38'!
lastTag
	"Returns a string which represents a hex-value of the 5 bytes tag ID last read by the sensor.
	This tag may or may not still be on the reader.
	If there was no tag read by the reader since attached, the method returns a string with 10 zeros.
	If the Phidget with this component is not attached return an empty string."
	
	| aString bytes |
	aString := ''.
	bytes := self lastTagBytes.
	bytes ifNotNil: [
		bytes do: [ :byte | aString := aString , byte hex ] ].
	^ aString! !

!PhdCrCompRFID methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:39'!
lastTagBytes
	"Returns a byte array with the 5 elements of the tag ID that was last read by the sensor.
	This tag may or may not still be on the reader.
	If there was no tag read by the reader since opened, the method returns a Byte array with zeros.
	If the Phidget is not attached return nil."
	
	phidget isAttached ifFalse: [ ^ nil ].
	^ self class primGetLastTag: phidget id! !

PhdCrComponent subclass: #PhdCrCompSimpleLED
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompSimpleLED commentStamp: 'lk 6/16/2009 19:46' prior: 0!
I am a special Phidget Component representing a simple LED.
I can be turned off or on (getter/setter exist).

Currently I only belong to PhdCrRFID.!

!PhdCrCompSimpleLED class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primIsOn: phidgetIdArray
	"Answer 1 if theLED is on, or other values otherwise."
	
	<primitive: 'primIsSimpleLEDOn' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompSimpleLED class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:51'!
primSetState: phidgetIdArray to: state
	"Set the state of the LED to true (state=1) or false (state=0).
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSimpleLEDState' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompSimpleLED methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:41'!
isOn
	"Answer true if the phidget is attached and the LED is on, else false."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class primIsOn: phidget id) > 0! !

!PhdCrCompSimpleLED methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:13'!
isOn: aBoolean 
	"Sets the state to true or false.
	true meens the LED is illuminated, false that it is not.
	returns true if setting was successful, and false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetState: phidget id
		to: anInt) == 1! !

!PhdCrComponent class methodsFor: 'instance creation' stamp: 'lk 6/16/2009 20:19'!
newWith: phidget
	"create a new Component with given phidget. This Phidget, once set, will never change again."

	^ self new
		setParent: phidget! !

!PhdCrComponent methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:35'!
on: aSymbol do: aBlock
	"When event aSymbol is signaled for my Phidget, do aBlock.
	This method is provided to have a moren natural interface.
	See PhdCrPhidget on:do: for more information."
	
	phidget on: aSymbol do: aBlock! !

!PhdCrComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 17:21'!
setParent: parent
	"Set my 'phidget' instance variable to the given one.
	If I already have a parent, do nothing."
	
	phidget ifNil: [ phidget := parent ].! !

PhdCrComponent subclass: #PhdCrMultiComponent
	instanceVariableNames: 'index nextLink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrMultiComponent commentStamp: 'lk 6/16/2009 18:56' prior: 0!
I am an abstract Phidget Component for components that cann exist serveral times on a Phidget, but the exact number differs from Phidget to Phidget. (The Phidget type allows different numbers).


Instance Variables
	index:		<Integer>
	nextLink:	<nil or PhdCrMultiComponent>

index
	- number that identifies me on a Phidgets that has more instances of my class. Like 'parent' it is provided on construction and will never change again.
	- Needed on every primitve invocation, except for the primitive getting the number of equal components (componentCount).

nextLink
	- Needed to be member of a linked list. Can be ignored for Phidget understanding.


About MultiComponents:
	When a Phidget with a multi-component is attached, it initializes the list containing instances of this class. It uses my classes 'getListFor:' method to do this.
	'getListFor:' dynamically gets the number of similar components and adds one list item for each to the list. It calls 'primComponentCount:' to determine this number. This method must be overritten in subclasses to point to the right primitive.
	A MultiComponent is created by sending the 'newWith:index:' method to my class, because like in simple Components it needs a phidget 'parent', and for Multicomponents it aditionally needs its index (see above).
!

PhdCrMultiComponent subclass: #PhdCrCompAnalogSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompAnalogSensor commentStamp: 'lk 6/16/2009 20:34' prior: 0!
I am a special Phidget Component for analog sensor values.
I can be asked for a value I am currently reading, from 0 to 1000 (getter exists).
My Phidget can call a user defined callback block when my value changes, when prior
	analogSensor on: #SensorChange do: [ "user defined content" ]
was called. Note: This block will only be evaluated if the change of the value is equal or bigger than my sensitivity.
My sensitivity is a value between 0 and 100. (getter/setter exist).

Currently I only belong to PhdCrInterfaceKit.!

!PhdCrCompAnalogSensor class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:55'!
primComponentCount: phidgetIdArray
	"Gets the number of sensors for a phidget analog sensors component."

	<primitive: 'primNumAnalogSensors' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:55'!
primGetSensitivity: phidgetIdArray at: index
	"Gets the current sensitivity of the given Phidget's sensor at given index."

	<primitive: 'primGetIntegerSensitivity' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:55'!
primGetValue: phidgetIdArray at: index
	"Gets the sensor value (0-1000) at the analog input with the given index."

	<primitive: 'primGetAnalogSensorValue' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:55'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an sensor value changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAnalogSensorChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAnalogSensor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:55'!
primSetSensitivity: phidgetIdArray at: index to: anInteger
	"Sets the sensitivity of given Phidget at given sensor index to the given value.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetIntegerSensitivity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAnalogSensor methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:13'!
sensitivity

	phidget isAttached ifFalse: [ ^ 0 ].
	^ self class
		primGetSensitivity: phidget id
		at: index - 1! !

!PhdCrCompAnalogSensor methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:13'!
sensitivity: anInteger
	"Sets the sensor change trigger of the sensor to 'anInteger'.
	a sensor change block will only be evaluated if the change is more than this trigger."
	
	phidget isAttached ifFalse: [ ^ false ].
	(anInteger isInRange: (0 to: 100)) ifFalse: [ ^ false ].

	^ (self class
		primSetSensitivity: phidget id
		at: index - 1
		to: anInteger) == 1! !

!PhdCrCompAnalogSensor methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:53'!
value
	"Gets the current value of the sensor (integer from 0 to 1000)."
	
	phidget isAttached ifFalse: [ ^ 0 ].
	^ self class
		primGetValue: phidget id
		at: index - 1! !

PhdCrMultiComponent subclass: #PhdCrCompAxis
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompAxis commentStamp: 'sp 6/19/2009 16:41' prior: 0!
I am a special Phidget Component for axes.
I can be asked for the current acceleration (from -3 to 3 (getter exists)) of an axis of me.
My Phidget can call a user defined callback block when my acceleration changes, when prior
	axis on: #AccelerationChange do: [ "user defined content" ]
was called. Note: This block will only be evaluated if the change of the value is equal or bigger than my sensitivity.
My sensitivity is a value between 0 and 1 (getter/setter exist).
By default it is set to 0.05, so that there will be no acceleration when I am not in move.

Currently I only belong to PhdCrAccelerometer.!

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'lk 6/21/2009 12:17'!
primAcceleration: phidgetHandle at: index
	"Gets the acceleration of the given axis."

	<primitive: 'primGetAxisAcceleration' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'lk 6/21/2009 12:16'!
primAccelerationMax: phidgetHandle at: index
	"Gets the max. acceleration on given axis"

	<primitive: 'primGetAxisAccelerationMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'lk 6/21/2009 12:16'!
primAccelerationMin: phidgetHandle at: index
	"Gets the min. acceleration on given axis"

	<primitive: 'primGetAxisAccelerationMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'multi-component' stamp: 'sp 6/18/2009 21:59'!
primComponentCount: phidgetHandle
	"Gets the number of axes for a phidget axis component."

	<primitive: 'primNumAxes' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/19/2009 09:25'!
primGetSensitivity: phidgetHandle at: index

	<primitive: 'primGetIndexedDoubleSensitivity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/18/2009 21:30'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an acceleration occurs on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAxisChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis class methodsFor: 'primitives' stamp: 'sp 6/19/2009 09:26'!
primSetSensitivity: phidgetHandle at: index to: anInteger

	<primitive: 'primSetIndexedDoubleSensitivity' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:18'!
acceleration
	"Gets the current acceleration of the axis (double between min and max).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primAcceleration: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:18'!
accelerationMax
	"Gets the max. acceleration of the axis.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primAccelerationMax: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:18'!
accelerationMin
	"Gets the min. acceleration of the axis.
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primAccelerationMin: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'pt 6/23/2009 18:44'!
angleDegrees

	^ self angleRadians radiansToDegrees! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'pt 6/23/2009 19:08'!
angleDegreesNumber 

	^ self angleDegrees + 90.0 / 180.0 * 1000.0! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'pt 6/23/2009 18:43'!
angleRadians

	| normalized |
	normalized := self acceleration.
	normalized < -1.0 ifTrue: [ normalized := -1.0 ].
	normalized > 1.0 ifTrue: [ normalized := 1.0 ].
	
	^ normalized arcSin! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:19'!
sensitivity
	"Gets the current sensitivity of the axis (double between 0.0 and 1.0).
	if there was an error, return 0.0"
	
	phidget isAttached ifFalse: [ ^ 0.0 ].
	^ self class
		primGetSensitivity: phidget id
		at: index - 1! !

!PhdCrCompAxis methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:22'!
sensitivity: aDouble
	"Sets the acceleration change trigger of the axis to 'aDouble'.
	a acceleration change block will only be evaluated if the change is more than this trigger.
	Return true if the operation succeeds, false otherwise."
	
	| value |
	phidget isAttached ifFalse: [ ^ false ].
	value := aDouble asFloat.
	( value >= 0.0 and: [ value <= 1.0 ] ) ifFalse: [ ^ false ].

	^ (self class
		primSetSensitivity: phidget id
		at: index - 1
		to: value) == 1! !

PhdCrMultiComponent subclass: #PhdCrCompDigitalInput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompDigitalInput commentStamp: 'lk 6/16/2009 20:34' prior: 0!
I am a special Phidget Component for digital input values.
I can be asked for a value I am currently reading, either true or false (getter exists). True means there is a current I can read, false there is not.
My Phidget can call a user defined callback block when my value changes, when prior
	digitalInput on: #InputChange do: [ "user defined content" ]
was called.

Currently I only belong to PhdCrInterfaceKit.!

!PhdCrCompDigitalInput class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:56'!
primComponentCount: phidgetIdArray
	"Gets the number of inputs for a phidget digital input component."

	<primitive: 'primNumDigitalInputs' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalInput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetValue: phidgetIdArray at: index
	"Gets the input value (0/1 = off/on) at the digital input with the given index."

	<primitive: 'primGetDigitalInputValue' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalInput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an input value changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexDigitalInputChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompDigitalInput methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:53'!
value
	"Gets the current value of the input (true/false = on/off)."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primGetValue: phidget id
		at: index - 1) == 1! !

PhdCrMultiComponent subclass: #PhdCrCompDigitalOutput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompDigitalOutput commentStamp: 'lk 6/16/2009 20:34' prior: 0!
I am a special Phidget Component for digital output values.
I can be asked for or set a value I am currently having, either true or false (getter/setter exist). True means there is a current at this output, false that there is not.
My Phidget can call a user defined callback block when my value changes, when prior
	digitalOutput on: #OutputChange do: [ "user defined content" ]
was called.

Currently I can belong to
	- PhdCrInterfaceKit
	- PhdCrRFID!

!PhdCrCompDigitalOutput class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:56'!
primComponentCount: phidgetIdArray
	"Gets the number of outputs for a phidget digital output component."

	<primitive: 'primNumDigitalOutputs' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalOutput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetValue: phidgetIdArray at: index
	"Gets the output value (0/1 = off/on) at the digital output with the given index."

	<primitive: 'primGetDigitalOutputValue' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompDigitalOutput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primSetSemaphoreIndexChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when an output value changes on the given Phidget.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexDigitalOutputChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompDigitalOutput class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primSetValue: phidgetIdArray at: index to: value
	"Sets the output value (0/1 = off/on) at the digital output with the given index to the given value.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetDigitalOutputValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompDigitalOutput methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:55'!
value
	"Gets the current value of the output (true/false = on/off)."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primGetValue: phidget id
		at: index - 1) == 1! !

!PhdCrCompDigitalOutput methodsFor: 'accessing' stamp: 'lk 6/16/2009 19:55'!
value: aBoolean
	"Sets the current value of the output (true = on, false = off).
	Returns true if the value was sucessfully set, false otherwise."
	
	| number |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean
		ifTrue: [ number := 1 ]
		ifFalse: [ number := 0 ].
	^ (self class
		primSetValue: phidget id
		at: index - 1
		to: number) == 1! !

PhdCrMultiComponent subclass: #PhdCrCompServoMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrCompServoMotor commentStamp: 'lk 6/16/2009 20:35' prior: 0!
I am a special Phidget Component for simple servo motors.
I have a current position, which can be retrieved and set (getter/setter exist). This position should be between my minimal and maximal position which can also be retrieved. Be careful that if those values cannot be retrieved (for instance if the Phidget s not attached) they return a value of 0.0, although this may also be a valid value.
My motor is automatically engaged when a new position is set. The boolean value whether it is engaged or not can be retrieved and set.
My Phidget can call a user defined callback block when my motor position changes, when prior
	servoMotor on: #PositionChange do: [ "user defined content" ]
was called.

Currently I only belong to PhdCrServo.!

!PhdCrCompServoMotor class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:56'!
primComponentCount: phidgetIdArray
	"Gets the number of servo motors for a (simple) servo motor component."

	<primitive: 'primNumServoMotors' module: 'PhidgetPlugin'>
	^ 0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetMaxPosition: phidgetIdArray at: index
	"Returns a float containing max. possible position of the motor at the given index."

	<primitive: 'primGetServoPositionMax' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetMinPosition: phidgetIdArray at: index
	"Returns a float containing min. possible position of the motor at the given index."

	<primitive: 'primGetServoPositionMin' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:56'!
primGetPosition: phidgetIdArray at: index
	"Returns a float containing the position of the motor at the given index."

	<primitive: 'primGetServoPosition' module: 'PhidgetPlugin'>
	^ 0.0! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primIsEngaged: phidgetIdArray at: index
	"Returns 1 if the motor on the given index is engaged, or other values otherwise."

	<primitive: 'primIsServoEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primSetEngaged: phidgetIdArray at: index to: value
	"Sets the engagement state (0/1 = off/on) of the motor at the given index.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetServoEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primSetPosition: phidgetIdArray at: index to: aFloat
	"Set the position of the motor at the given index to aFloat. aFloat must be between primGetPositionMin and primGetPositionMax.
	Return 1 if the operation succeeded, other values otherwise."

	<primitive: 'primSetServoPosition' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor class methodsFor: 'primitives' stamp: 'lk 6/16/2009 20:57'!
primSetSemaphoreIndexPositionChange: semaphoreIndex for: phidgetIdArray
	"registers the given Semaphore to be signaled when a motor changes its position.
	Return 1 if the operation succeeded, other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexServoPositionChange' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 15:10'!
isEngaged
	"Returns the engaged state of the motor. This is whether the motor is powered or not."
	
	phidget isAttached ifFalse: [ ^ false ].
	^ (self class
		primIsEngaged: phidget id
		at: index - 1) == 1! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/21/2009 12:12'!
isEngaged: aBoolean
	"Sets the engaged state of a motor. This is whether the motor is powered or not.
	Returns true if the setting was successful, false otherwise."
	
	| anInt |
	phidget isAttached ifFalse: [ ^ false ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	^ (self class
		primSetEngaged: phidget id
		at: index - 1
		to: anInt) == 1! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:10'!
position
	"Returns the current position of the motor as a floating point number.
	This value is between self positinMin and self positionMax.
	If it cannot be retrieved it is 0.0 (although this may also be a valid value)."
	
	| result |
	phidget isAttached ifFalse: [ ^ 0.0 ].
	result := self class
		primGetPosition: phidget id
		at: index - 1.
	^ result
		ifNotNil: [ result ]
		ifNil: [ 0.0 ]! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:11'!
position: aValue
	"Set the position of the motor.
	aValue should be a number betwenn self positionMin and self positionMax (inclusivly).
	Returns true if the setting was successful, false otherwise."
	
	| aFloat |
	phidget isAttached ifFalse: [ ^ false ].
	aFloat := aValue asFloat.
	(self positionIsInRange: aFloat) ifFalse: [ ^ false ].
	^ (self class
		primSetPosition: phidget id
		at: index - 1
		to: aFloat) == 1! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:12'!
positionIsInRange: position
	"Return true if the position is in range of this motor, false otherwise.
	The position is in range if the Phidget is attached and the position is between the min. and max. position."
	
	phidget isAttached ifFalse: [ ^ false ].
	self positionMin <= position ifFalse: [ ^ false ].
	self positionMax >= position ifFalse: [ ^ false ].
	^ true! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:12'!
positionMax
	"Returns the max. allowed motor position for this motor.
	If it cannot be retrieved it is 0.0 (although this may also be a valid value)."
	
	| result |
	phidget isAttached ifFalse: [ ^ 0.0 ].
	result := self class
		primGetMaxPosition: phidget id
		at: index - 1.
	^ result
		ifNotNil: [ result ]
		ifNil: [ 0.0 ]! !

!PhdCrCompServoMotor methodsFor: 'accessing' stamp: 'lk 6/16/2009 20:13'!
positionMin
	"Returns the min. allowed motor position for this motor.
	If it cannot be retrieved it is 0.0 (although this may also be a valid value)."
	
	| result |
	phidget isAttached ifFalse: [ ^ 0.0 ].
	result := self class
		primGetMinPosition: phidget id
		at: index - 1.
	^ result
		ifNotNil: [ result ]
		ifNil: [ 0.0 ]! !

!PhdCrCompServoMotor methodsFor: 'prototyp' stamp: 'pt 6/23/2009 18:50'!
positionNumber: aNumber

	| max min |
	( 0 <= aNumber and: [ aNumber <= 1000 ] ) ifFalse: [ ^ self ].
	max := self positionMax.
	min := self positionMin.
	min == -1000 ifFalse: [
		self position: max - min * (aNumber asFloat) / 1000.0 + min ].! !

!PhdCrMultiComponent class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:39'!
componentCount: phidget
	"Returns the number of components in a multicomponent phidget.
	Uses a primitive the subclasses must define. This primitive can for example do a call to get the number of sensors on an InterfaceKit.
	If there was an error getting the value, 0 is returned."
	
	phidget isAttached ifTrue: [ | result |
		result := self primComponentCount: phidget id.
		result ifNil: [ ^ 0 ].
		^ result ].
	 ^ 0! !

!PhdCrMultiComponent class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:40'!
getListFor: phidget
	"Answers a LinkedList of instances of my class, with self componentCount elements, each initialized with its index in the list."

	| list size |
	list := LinkedList new.
	size := self componentCount: phidget.
	
	(1 to: size) do: [ :i |
		list add: (self newWith: phidget index: i) ].
	^ list! !

!PhdCrMultiComponent class methodsFor: 'instance creation' stamp: 'lk 6/16/2009 20:38'!
newWith: phidget index: index
	"Create and return a new MultComponent with given Phidget 'parent' and index.
	Once those values are set, they are never changed again."

	^ (self newWith: phidget)
		setIndex: index! !

!PhdCrMultiComponent class methodsFor: 'multi-component' stamp: 'lk 6/16/2009 20:52'!
primComponentCount: phidgetIdArray
	"Gets the number of components for a phidget multi component.
	This can for instance be a primitive to get the number of sensors on an InterfaceKit."

	self subclassResponsibility.! !

!PhdCrMultiComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 18:38'!
nextLink

	^ nextLink! !

!PhdCrMultiComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 18:38'!
nextLink: aMultiComponent

	nextLink := aMultiComponent.! !

!PhdCrMultiComponent methodsFor: 'accessing' stamp: 'lk 6/1/2009 18:17'!
setIndex: aNumber
	"Set my 'index' instance variable to the given one.
	If I already have an index, do nothing."
	
	index ifNil: [ index := aNumber ].! !

Object subclass: #PhdCrPhidget
	instanceVariableNames: 'opened semaphores userProcessBlocks id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

PhdCrPhidget class
	instanceVariableNames: 'SystemCallbacks'!

!PhdCrPhidget commentStamp: 'lk 6/16/2009 16:17' prior: 0!
I am an abstract class that provides some primitives and methods for Phidget support. Please read the comments of my subclasses to find more information how to use Phidgets in Squeak.

Instance Variables
	id:						<ByteArray>
	opened:					<Boolean>
	userProcessBlocks:	<Dictionary> (Symbol -> Block)
	semaphores:				<Dictionary> (Symbol -> Semaphore)
Class Instance Variables
	SystemCallback:		<Dictionary> (Symbol -> Symbol)
	
id
	- a ByteArray that contains the Phidget's unique identifier for the C API. Every primitive (except the one where the handle is created) needs this ByteArray to let the C API know about which Phidget it is currently talking to.
	- the id is obtained when creating the phidget object and will stay there until the object is deleted or the id became invalid (see checkInvalid method).
	- an id can become invalid if the phidget is created, the Squeak image saved, closed and then reloaded. As the plugin was shutdown when the VM is closed, the hardware connection to the phidget is broken. checkValid checks the id and resets it to a default value if the id has become invalid. This object cannot be used for communication with any Phidget anymore.
	- checkValid should be called before doing any primitve with a phidget id.

opened
	- true, if the Phidget was opened - Squeak is able to communicate with the hardware. This does not mean there is a Phidget attached.
	- false, if the Phidgets was not opened or was opened and then closed.

semaphores
	- A Dictionary with entries of the following form: aSymbol -> aSemaphore.
	aSymbol may be #Attach, #Detach, #Error for Phidgets in general. Other symbols may exist for phidget specific support in my subclasses. My classes method 'knownCallback' answers a list of callbacks I know.
	I use this dictionary to handle the callbacks from the Phidget C API.
	These semaphores can be registered for certain events (like the attach event) - so that the C API signals is if the event occours. These semaphores are used in my callbackProcess method.

userProcessBlocks
	- a Dictionary with entries of the following form: aSymbol -> aBlock.
	The symbols are equivalent to the symbols used in the semaphores dictionary.
	I use this dictionary to handle the callbacks from the Phidget C API.
	Each time a semaphore from the semaphores dictionary was signaled from the C API, I evaluate the corresponding block from this dictionary in the callbackProcess method.
	In the base class possible callbacks are: #Attach, #Detach, #Error

SystemCallbacks
	My class has a method called 'systemCallbacks'. This Dictionary is similar to the userProcessBlocks: It maps from callbackNames to message selectors. Those messages willbe sent when the semaphore at 'callbackName' is signaled.
	In contrast to userProcessBlocks this Dictionary is immutable to the user. Currently it is only used for initializing and deleting phidget components an attaching/detaching (see below).

About Components:
	A phidget can have several components, e.g. a list of outputs or a LED. Those components derive from the PhdCrComponent class. See its documentation to lern more about them.
	When a phidget is attached, it initializes its components using 'initializeComponents'. When the phidget is detached or closed, 'deleteComponents' deletes them again.
	A Component a Phidget can have is represented by an instance variable in that class. Add a getter and initialisation/deletion instructions in initialize / initializeComponents / deleteComponents.
	As there are different Phidgets per phidget class, the umber of components can differ. A InterfaceKit can for instance have 8 sensors, others might have none. The implementations of initializeComponents, delete components and initialize must be aware of that.

IMPORTANT:
After using the Phidget it is important to close the connection to the hardware by sending 'close':
	phidget close.	!

PhdCrPhidget class
	instanceVariableNames: 'SystemCallbacks'!

PhdCrPhidget subclass: #PhdCrAccelerometer
	instanceVariableNames: 'axes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrAccelerometer commentStamp: 'sp 6/19/2009 16:19' prior: 0!
I am a representation of a Phidget called 'Accelerometer' in Squeak.
To use a Accelerometer, create a new instance and open a connection to the hardware board with:
	accel := PhdCrInterfaceKit new.
	accel openAny.
Send me a block to be evaluated every time the hardware board was attached, detached, an error or acceleration occoured.
For example:
	accel
		on: #Attach do: [ Transcript show:'accel was attached!!;cr' ];
		on: #Detach do: [ Transcript show:'accel was detached!!;cr' ];
		on: #AccelerationChange do: [ Transcript show:'A axis changed it's value!!' ].


Instance Variables
	axes:		<LinkedList of PhdCrCompDigitalOutput's>

axes:
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrComponent's as the Phidget has. When the Phidget is detached the list is emptied. See the instance's class documentation for more information.


new callbacks:
	#AccelerationChange (an axis has occured an acceleration)!

!PhdCrAccelerometer class methodsFor: 'callbacks' stamp: 'sp 6/18/2009 17:42'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Accelerometer knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #AccelerationChange )! !

!PhdCrAccelerometer class methodsFor: 'primitives' stamp: 'sp 6/18/2009 17:43'!
primCreate
	"Create a new the Acceleration object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primAccelerometerCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrAccelerometer methodsFor: 'accessing' stamp: 'sp 6/18/2009 17:45'!
axes
	^ axes! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 17:40'!
deleteComponents
	"This method is called when the Accelerometer is detached. It delets previously in 'initializeComponents' built component axes."

	axes := LinkedList new.! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 17:36'!
initialize
	"additionally reset my components."

	super initialize.
	
	axes := LinkedList new.! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 23:23'!
initializeComponents
	"This method is called when an Accelerometer is attached. It initializes its component axes.
	The component will be deleted in 'deleteComponents' on detaching."

	axes := PhdCrCompAxis getListFor: self.! !

!PhdCrAccelerometer methodsFor: 'initialize-open-close' stamp: 'sp 6/18/2009 21:19'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for
	#AxisChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #AccelerationChange).
	PhdCrCompAxis
		primSetSemaphoreIndexChange: semaIndex
		for: id.! !

PhdCrPhidget subclass: #PhdCrInterfaceKit
	instanceVariableNames: 'sensors inputs outputs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrInterfaceKit commentStamp: 'sp 6/19/2009 16:13' prior: 0!
I am a representation of a Phidget called 'Interface Kit' in Squeak.
To use a Interface Kit, create a new instance and open a connection to the hardware board with:
	ifKit := PhdCrInterfaceKit new.
	ifKit openAny.
Send me a block to be evaluated every time the hardware board was attached, detached, an error occoured, a sensor value was changed, an input value was changed or an output value was changed. For example:
	ifKit
		on: #Attach do: [ Transcript show:'ifKit was attached!!;cr' ];
		on: #Detach do: [ Transcript show:'ifKit was detached!!;cr' ];
		on: #SensorChange do: [ Transcript show:'A sensor changed it's value!!' ].


Instance Variables
	sensors:		<LinkedList of PhdCrCompAnalogSensor's>
	inputs:		<LinkedList of PhdCrCompDigitalInput's>
	outputs:		<LinkedList of PhdCrCompDigitalOutput's>

sensors / inputs / outputs: (each)
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrComponent's as the Phidget has. When the Phidget is detached the list is emptied. See the instance's class documentation for more information.


new callbacks:
	#SensorChange (a sensor has changed its value),
	#InputChange (an input has changed its value),
	#OutputChange (an output has changed its value)
!

!PhdCrInterfaceKit class methodsFor: 'callbacks' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the InterfaceKit knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #SensorChange #InputChange #OutputChange )! !

!PhdCrInterfaceKit class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:17'!
primCreate
	"Create a new the Interface Kit object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primInterfaceKitCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 13:36'!
deleteComponents
	"This method is called when the InterfaceKit is detached. It delets previously in 'initializeComponents' built components: sensors, inputs, outputs."

	sensors := LinkedList new.
	inputs := LinkedList new.
	outputs := LinkedList new.! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:04'!
initialize
	"additionally reset my components."

	super initialize.
	
	inputs := LinkedList new.
	outputs := LinkedList new.
	sensors := LinkedList new.! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 14:59'!
initializeComponents
	"This method is called when an InterfaceKit is attached. It initializes its components: sensors, inputs, outputs.
	The components will be deleted in 'deleteComponents' on detaching."
	
	sensors := PhdCrCompAnalogSensor getListFor: self.
	inputs := PhdCrCompDigitalInput getListFor: self.
	outputs := PhdCrCompDigitalOutput getListFor: self.! !

!PhdCrInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:13'!
inputs
	^ inputs! !

!PhdCrInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:13'!
outputs
	^ outputs! !

!PhdCrInterfaceKit methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 16:07'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for #SensorChange, #InputChange and #OutputChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #SensorChange).
	PhdCrCompAnalogSensor
		primSetSemaphoreIndexChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #InputChange).
	PhdCrCompDigitalInput
		primSetSemaphoreIndexChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #OutputChange).
	PhdCrCompDigitalOutput
		primSetSemaphoreIndexChange: semaIndex
		for: id.! !

!PhdCrInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:14'!
sensors
	^ sensors! !

!PhdCrPhidget class methodsFor: 'callbacks' stamp: 'lk 6/16/2009 21:14'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Phidget knows. A subclass may extend this list if needed."
	
	^ #( #Attach #Detach #Error )! !

!PhdCrPhidget class methodsFor: 'accessing' stamp: 'lk 6/16/2009 21:15'!
lastError
	"Returns the error code of the last error in the PhidgetPlugin, or nil if something went wrong.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h in line 2250 ff."

	^ self primLastError! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:15'!
primClose: phidgetidArray
	"Close the Phidget with given id. 
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primClose' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:14'!
primIsAttached: phidgetidArray
	"Returns 1 if the Phidget is attached, and other values otherwise."
	
	<primitive: 'primIsAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:15'!
primIsValid: phidgetidArray
	"Returns 1 if the Phidget id is valid, or other values otherwise."
	
	<primitive: 'primIsValid' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:15'!
primLastError
	"Returns the error code of the last error in the Plugin, or nil if something went wrong.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h in line 2250 ff."

	<primitive: 'primLastError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primOpen: phidgetidArray withSerial: serial
	"Opens the Phidget with the given serial number.
	Return 1 if the operation succeeded, or other values otherwise."

	<primitive: 'primOpen' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primSetSemaphoreIndexAttach: semaphoreIndex for: phidgetidArray
	"registers the given Semaphore to be signaled when the Phidget with given id is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexAttach' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primSetSemaphoreIndexDetach: semaphoreIndex for: phidgetidArray
	"registers the given Semaphore to be signaled when the Phidget with given id is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexDetach' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:16'!
primSetSemaphoreIndexError: semaphoreIndex for: phidgetidArray
	"registers the given Semaphore to be signaled when the Phidget with given id is attached.
	Return 1 if the operation succeeded, or other values otherwise."
	
	<primitive: 'primSetSemaphoreIndexError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget class methodsFor: 'callbacks' stamp: 'lk 6/1/2009 17:04'!
systemCallbacks
	"This Method answers a Doctionary similar to the instance's userProcessBlocks.
	It may for instance contain message selectors sent when attaching or detaching a phidget, so the phidget components can be initialized."
	
	SystemCallbacks ifNotNil: [ ^ SystemCallbacks ].
	
	SystemCallbacks := Dictionary new.
	SystemCallbacks
		add: #Attach -> #initializeComponents;
		add: #Detach -> #deleteComponents.
	^ SystemCallbacks! !

!PhdCrPhidget methodsFor: 'callbacks' stamp: 'lk 6/16/2009 21:07'!
callbackProcess: callbackName 
	"This method will be forked as an extra process for every callback that a phidget needs.
	The semaphores were previously initialised and registered as external objects.
	self class knownCallbacks lists all possible semaphore names.
	systemCallbacks contains message selectors to be sent."
	
	(semaphores at: callbackName) wait.
	
	[ self isOpen ] whileTrue: [
		(self class systemCallbacks includesKey: callbackName) 
			ifTrue: [ self perform: (self class systemCallbacks at: callbackName) ].
		(userProcessBlocks at: callbackName) value.
		
		(semaphores at: callbackName) wait
	]! !

!PhdCrPhidget methodsFor: 'testing' stamp: 'lk 6/15/2009 13:46'!
checkValid
	"Checks if the Phidgets has an id and if this id is valid."

	^ id isNil not and: [ | result |
		result := (self class primIsValid: id) == 1.
		result ifFalse: [
			"this may have happened if a Phidget was created and then the image was saved,
			closed and reloaded. The hardware connection is broken. We need to make this
			instance fully invalid."
			id := nil.
			self close ].
		result ]! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:07'!
close
	"Close the Phidget."
	
	self isOpen ifFalse: [ ^ self ].	"already closed"
	
	self class primClose: id.
	opened := false.
	self
		unregisterSemaphores;
		deleteComponents.! !

!PhdCrPhidget methodsFor: 'callbacks' stamp: 'lk 6/1/2009 00:47'!
createCallbackProcessBlock: callbackName
	"Create a callback process block to be forked.
	Must be swapped out to work in loops."

	^ [ self callbackProcess: callbackName ]! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/1/2009 12:41'!
deleteComponents
	"This method is called when a Phidget is detached. It should delete previously in 'initializeComponents' built components, e.g. outputs, sensors, ...
	The abstract phidget does not have components, so it does nothing here."! !

!PhdCrPhidget methodsFor: 'callbacks' stamp: 'lk 6/1/2009 12:05'!
forkCallbacks
	"Fork the callback processes."
	
	self class knownCallbacks do: [ :callbackName |
		(self createCallbackProcessBlock: callbackName) fork ].! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 6/15/2009 11:24'!
id
	^ id! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:06'!
initialize
	"Initialise the callbacks that are used in every Phidget.
	Every subclass should call super initialise or initialise the callbacks by their own.
	In addition mark phidget as opened and create the representation in the plugin using a primitive."
	
	super initialize.
	
	semaphores := Dictionary new.
	userProcessBlocks := Dictionary new.
	opened := false.
	
	self class knownCallbacks do: [ :name | 
		semaphores add: name -> Semaphore new.
		userProcessBlocks add: name -> [] ].
	
	id := self class primCreate.! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/1/2009 12:41'!
initializeComponents
	"This method is called when a Phidget is attached. It should initialize its components (sensors, outputs, ...).
	The components will be deleted in 'deleteComponents' on detaching.
	The abstract phidget does not have components, so it does nothing here."! !

!PhdCrPhidget methodsFor: 'testing' stamp: 'lk 6/15/2009 20:34'!
isAttached
	"Returns true if the Phidget is attached, and false otherwise."

	^ self checkValid & opened and: [ (self class primIsAttached: id) == 1 ]! !

!PhdCrPhidget methodsFor: 'testing' stamp: 'lk 6/16/2009 21:09'!
isOpen
	"return true if I am open (having a hardware connection to a Phidget or waitng for one).
	Also make sure I am sill valid."

	^ self checkValid and: [ opened ]! !

!PhdCrPhidget methodsFor: 'accessing' stamp: 'lk 6/16/2009 21:09'!
on: callbackName do: aBlock 
	"aBlock will be evaluated when the semaphore at callbackName is signaled.
	self class knownCallbacks lists all possible callback names."
	
	(self class knownCallbacks includes: callbackName) ifTrue: [
		userProcessBlocks 
			at: callbackName
			put: aBlock ].! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 13:09'!
openAny
	"opens hardware connection to any phidget with matching class."

	^ self openWithSerial: -1! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:10'!
openWithSerial: serialNumber 
	"Opens the Phidget with the given serial number.
	Opens any Phidgets of matching class if serialNumber is -1."

	(self checkValid and: [self isOpen not]) ifFalse: [ ^ self ].
	
	self registerSemaphores.
	self class
		primOpen: id
		withSerial: serialNumber.
	opened := true.
	self forkCallbacks.! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 12:32'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. May be extended in subclasses."
	
	| semaIndex |
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #Attach).
	self class
		primSetSemaphoreIndexAttach: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #Detach).
	self class
		primSetSemaphoreIndexDetach: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #Error).
	self class
		primSetSemaphoreIndexError: semaIndex
		for: id.! !

!PhdCrPhidget methodsFor: 'testing' stamp: 'lk 6/16/2009 21:10'!
testSemaphoresReset
	"Answer true if the semaphores are reset, false otherwise."

	id do: [ :byte | byte isZero ifFalse: [ ^ false ] ].
	^ true! !

!PhdCrPhidget methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:11'!
unregisterSemaphores
	"Unregister the semaphores which were previously registered with 'self registerSemaphores'."

	semaphores do: [ :sema |
		"If I am not opened and the semaphore is signaled, the callback process will terminate."
		opened := false.
		sema signal.
		
		Smalltalk unregisterExternalObject: sema
	].! !

PhdCrPhidget subclass: #PhdCrRFID
	instanceVariableNames: 'rfid led outputs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrRFID commentStamp: 'lk 6/16/2009 16:39' prior: 0!
I am a representation of a Phidget called 'Phidget RFID' in Squeak.
To use the RFID Phidget, create a new instance and open a connection to the hardware board with:
	rfid := PhdCrRFID new.
	rfid openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured, an output value was changed, a tag was found by the RFID reader, or if the tag was lost. For example:
	rfid
		on: #Attach do: [ Transcript show:'rfid was attached!!';cr ];
		on: #Detach do: [ Transcript show:'rfid was detached!!';cr ];
		on: #TagFound do: [ Transcript show:'rfid found a tag';cr ].


Instance Variables
	rfid:			<PhdCrCompRFID>
	outputs:		<LinkedList of PhdCrCompDigitalOutput's>
	led:			<PhdCrCompSimpleLED>

rfid:
	- a PhdCrCompRFID needed to get tag information of the RFID Phidget. See the PhdCrCompRFID class documentation for more information.

outputs:
	- a LinkedList. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrCompDigitalOutput's as the Phidget has. When the Phidget is detached the list is emptied. See the PhdCrCompDigitalOutput class documentation for more information.

led:
	- a PhdCrCompSimpleLED that is on-board on this type of Phidget. See the PhdCrCompSimpleLED class documentation for more information.


new callbacks:
	#TagFound (a tag was found by the rfid sensor),
	#TagLost (a tag was found by the rfid sensor),
	#OutputChange (an output has changed its value)


example indicating if a tag was found using the on-board LED
	rfid := PhdCrRFID new.
	rfid
		on: #Attach do: [ rfid rfidSensor setAntennaState: true ];
		on: #TagFound do: [ rfid led setState: true ];
		on: #TagLost do: [ rfid led setState: false ];
		openAny.
	"Now attach a RFID Phidget, hold a tag near the Phidget and then take it away."
	rfid close.!

!PhdCrRFID class methodsFor: 'callbacks' stamp: 'lk 5/30/2009 20:03'!
knownCallbacks
	"Returns a list of callback names (#symbols) the RFID Phidget knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #OutputChange #TagFound #TagLost )! !

!PhdCrRFID class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:18'!
primCreate
	"Create a new the RFID object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primRFIDCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 20:31'!
deleteComponents
	"This method is called when the RFID Phidget is detached. It delets previously in 'initializeComponents' built components: outputs"

	outputs := LinkedList new.! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:11'!
initialize
	"additionally reset my components."

	super initialize.
	
	rfid := PhdCrCompRFID newWith: self.
	led := PhdCrCompSimpleLED newWith: self.
	outputs := LinkedList new.! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 20:31'!
initializeComponents
	"This method is called when a RFID Phidget is attached. It initializes its components:  outputs.
	The components will be deleted in 'deleteComponents' on detaching."

	outputs := PhdCrCompDigitalOutput getListFor: self.! !

!PhdCrRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:21'!
led
	^ led! !

!PhdCrRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 16:15'!
outputs
	^ outputs! !

!PhdCrRFID methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 17:31'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for #OutputChange, #TagFound and #TagLost."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #OutputChange).
	PhdCrCompDigitalOutput
		primSetSemaphoreIndexChange: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #TagFound).
	PhdCrCompRFID
		primSetSemaphoreIndexTagFound: semaIndex
		for: id.
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #TagLost).
	PhdCrCompRFID
		primSetSemaphoreIndexTagLost: semaIndex
		for: id.! !

!PhdCrRFID methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:20'!
rfidSensor
	^ rfid! !

PhdCrPhidget subclass: #PhdCrServo
	instanceVariableNames: 'motors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrServo commentStamp: 'sp 6/19/2009 16:15' prior: 0!
I am a representation of a Phidget called 'Servo' (here: 'Servo board') in Squeak.
To use the Servo board, create a new instance and open a connection to the hardware board with:
	servo := PhdCrServo new.
	servo openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured or the position of a motor was changed. For example:
	servo
		on: #Attach do: [ Transcript show:'servo was attached!!;cr' ];
		on: #Detach do: [ Transcript show:'servo was detached!!;cr' ];
		on: #PositionChange do: [ Transcript show:'A motor changed it's position!!' ].


Instance Variables
	motors:		<LinkedList of PhdCrCompServoMotor's>

motors:
	- a LinkedList of PhdCrCompServoMotor's. This list is empty if no Phidget is attached. When a Phidget is attached this list is automatically filled with as many PhdCrCompServoMotor's as the Phidget has. When the Phidget is detached the list is emptied. See the PhdCrCompServoMotor class documentation for more information.


new callbacks:
	#PositionChange (a motor has changed its position)
!

!PhdCrServo class methodsFor: 'callbacks' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Servo knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #PositionChange )! !

!PhdCrServo class methodsFor: 'primitives' stamp: 'lk 6/16/2009 21:18'!
primCreate
	"Create a new the Servo object. Returns a new id for the Phidget, or nil if there was an error."

	<primitive: 'primServoCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 21:40'!
deleteComponents
	"This method is called when the Phidget is detached. It resets previously in 'initializeComponents' built components: motors"

	motors := LinkedList new.! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/16/2009 21:12'!
initialize
	"additionally reset my components."

	super initialize.
	
	motors := LinkedList new.! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 21:41'!
initializeComponents
	"This method is called when the Phidget is attached. It initializes its components:  motots.
	The components will be reset in 'deleteComponents' on detaching."

	motors := PhdCrCompServoMotor getListFor: self.! !

!PhdCrServo methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:37'!
motors
	^ motors! !

!PhdCrServo methodsFor: 'initialize-open-close' stamp: 'lk 6/15/2009 22:16'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding a semaphore for #PositionChange."
	
	| semaIndex |
	super registerSemaphores.
	
	semaIndex := Smalltalk registerExternalObject: (semaphores at: #PositionChange).
	PhdCrCompServoMotor
		primSetSemaphoreIndexPositionChange: semaIndex
		for: id.! !
